<?php
/*
  Plugin Name: Pretty Sidebar Categories
  Plugin URI: http://grandslambert.tk/plugins/pretty-sidebar-categories.html
  Description: Display posts from selected categories in your sidebars. Multiple formats including featured images are possible.
  Author: GrandSlambert
  Version: 1.2
  Author: grandslambert
  Author URI: http://grandslambert.tk/


 * *************************************************************************

  Copyright (C) 2011-2013 GrandSlambert

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

 * *************************************************************************

 */

/* Class Declaration */

class PrettySidebarCategories extends WP_Widget {

	var $version = '1.2';

	/* Options page name */
	var $optionsName = 'pretty-sidebar-categories-options';
	var $menuName = 'pretty-sidebar-categories-settings';
	var $pluginName = 'Sidebar Categories';
	var $options = array( );

	/* Widget constructor */

	function PrettySidebarCategories() {
		/* Load Langague Files */
		add_action( 'init', array( &$this, 'init' ) );

		/* Plugin Settings */
		/* translators: The name of the plugin, should be a translation of "Pretty Sidebar Categories" only! */
		$this->pluginName = __( 'Pretty Sidebar Categories', 'pretty-sidebar-categories' );

		/* translators: The description of the Sidebar Categories widget on the Appearance->Widgets page. */
		$widget_ops = array( 'description' => __( 'List posts from selected categories on your sidebar. By GrandSlambert.', 'pretty-sidebar-categories' ) );
		$control_ops = array( 'width' => 400, 'height' => 350 );
		/* translators: The title for the widget. */
		$this->WP_Widget( 'sidebar_categories', $this->pluginName, $widget_ops, $control_ops );

		/* Plugin Folders */
		$this->pluginPath = WP_PLUGIN_DIR . '/' . basename( dirname( __FILE__ ) );
		$this->pluginURL = WP_PLUGIN_URL . '/' . basename( dirname( __FILE__ ) );
		$this->templatesPath = WP_PLUGIN_DIR . '/' . basename( dirname( __FILE__ ) ) . '/templates/';
		$this->templatesURL = WP_PLUGIN_URL . '/' . basename( dirname( __FILE__ ) ) . '/templates/';
		$this->load_settings();

		/* Setup actions */
		add_action( 'wp_loaded', array( &$this, 'wp_loaded' ) );
		add_action( 'wp_print_styles', array( &$this, 'wp_print_styles' ) );
		add_action( 'admin_init', array( &$this, "admin_init" ) );
		add_action( 'admin_menu', array( &$this, "admin_menu" ) );
		add_action( 'admin_print_scripts', array( &$this, 'admin_print_scripts' ) );

		/* Setup Filters */
		add_filter( 'plugin_action_links', array( &$this, "plugin_action_links" ), 10, 2 );
		add_filter( 'whitelist_options', array( &$this, 'whitelist_options' ) );

		/* Add Image Sizes */
		add_image_size( 'sidebar-image-small', $this->options['small-image'][0], $this->options['small-image'][1], $this->options['hard-crop'] );
		add_image_size( 'sidebar-image-full', $this->options['full-image'][0], $this->options['full-image'][1], $this->options['hard-crop'] );
	}

	function load_settings() {
		$options = get_option( $this->optionsName );

		$defaults = array(
			'default-style' => 'list',
			'small-image' => array( 100, 75 ),
			'full-image' => array( 200, 125 ),
			'hard-crop' => isset( $options['hard-crop'] ) ? $options['hard-crop'] : true,
			'hide_empty' => false,
			'post_types' => function_exists( 'get_post_types' ) ? get_post_types( array( 'public' => true ) ) : array( 'post', 'page' )
		);

		$this->options = wp_parse_args( $options, $defaults );
	}

	/**
	 * Add all options to the whitelist for the NONCE
	 * Required for Wordpress MU support
	 */
	function whitelist_options( $whitelist ) {
		if ( is_array( $whitelist ) ) {
			$option_array = array( $this->pluginName => $this->optionsName );
			$whitelist = array_merge( $whitelist, $option_array );
		}

		return $whitelist;
	}

	function init() {
		load_plugin_textdomain( 'pretty-sidebar-categories', false, dirname( plugin_basename( __FILE__ ) ) . '/language' );
	}

	/**
	 * Register the settings menu.
	 */
	function admin_menu() {
		add_options_page( $this->pluginName . ' ' . __( 'Settings', 'pretty-sidebar-categories' ), __( 'Sidebar Categories', 'pretty-sidebar-categories' ), 'manage_options', $this->menuName, array( &$this, 'options_panel' ) );
	}

	/**
	 * Display the settings page.
	 */
	function options_panel() {
		include($this->pluginPath . "/includes/options-panel.php");
	}

	/**
	 * Adds a settings link next to Login Configurator on the plugins page
	 */
	function plugin_action_links( $links, $file ) {
		static $this_plugin;

		if ( !$this_plugin ) {
			$this_plugin = plugin_basename( __FILE__ );
		}

		if ( $file == $this_plugin ) {
			$settings_link = '<a href="' . admin_url() . '/options-general.php?page=' . $this->menuName . '">' . __( 'Settings', 'pretty-sidebar-categories' ) . '</a>';
			array_unshift( $links, $settings_link );
		}

		return $links;
	}

	/**
	 * Register the options
	 */
	function admin_init() {
		register_setting( $this->optionsName, $this->optionsName );
		wp_register_script( 'pretty-sidebar-categories-script-admin', $this->pluginURL . '/includes/pretty-sidebar-categories-admin.js' );
	}

	/**
	 * Register the style sheet used by the widget.
	 */
	function wp_loaded() {
		wp_register_style( 'pretty-sidebar-categories-style', $this->get_template( 'pretty-sidebar-categories', '', '.css', 'url' ) );
	}

	/**
	 * Output the style sheet.
	 */
	function wp_print_styles() {
		wp_enqueue_style( 'pretty-sidebar-categories-style' );
	}

	/**
	 * Output the adminisitration javascripts.
	 */
	function admin_print_scripts() {
		wp_enqueue_script( 'pretty-sidebar-categories-script-admin' );
	}

	/**
	 * Get the defaults for the form and widget.
	 *
	 * @param array $args
	 * @return array
	 */
	function get_defaults( $args = array( ) ) {
		$defaults = array(
			'taxonomy' => 'category',
			'style' => 'list',
			'title' => '',
			'cat' => false,
			'link_cat' => false,
			'num_posts' => 5,
			'post_types' => $this->options['post_types'],
			'orderby' => 'title',
			'order' => 'asc',
			'match_single_page' => false,
			'show_description' => false,
			'show_thumbnail' => false,
			'thumbnail_size' => 'thumbnail',
			'show_date' => false,
			'show_excerpt' => false,
			'excerpt_length' => 20,
			'excerpt_more_separator' => '&raquo;',
			'excerpt_more' => __( 'read more', 'pretty-sidebar-categories' ),
			'link_excerpt_more' => false,
			'hide_current' => false
		);

		return wp_parse_args( $args, $defaults );
	}

	/**
	 * Set up and display the widget form.
	 *
	 * @global array $_wp_additional_image_sizes
	 * @param array $instance
	 */
	function form( $instance ) {
		global $_wp_additional_image_sizes;
		$instance = $this->get_defaults( $instance );

		$taxonomies = get_taxonomies();

		include ($this->pluginPath . '/includes/widget-form.php');
	}

	/**
	 * Process the widget form
	 * @param array $new_instance
	 * @param array $old_instance
	 * @return array
	 */
	function update( $new_instance, $old_instance ) {
		return $new_instance;
	}

	/**
	 * Create the widget display
	 *
	 * @param array $args
	 * @param array $instance
	 * @return boolean
	 */
	function widget( $args, $instance ) {
		global $post;
		if ( isset( $instance['error'] ) && $instance['error'] ) {
			return;
		}

		extract( $args, EXTR_SKIP );
		$instance = $this->get_defaults( $instance );
		$taxonomy = $instance['taxonomy'];

		/* Get the list of terms to search for */
		if ( $instance[$taxonomy] == 0 ) {
			if ( $instance['match_single_page'] ) {
				$post_terms = get_the_terms( get_the_id(), $taxonomy );
				$terms = array( );

				if ( is_array( $post_terms ) ) {
					foreach ( $post_terms as $term ) {
						array_push( $terms, $term->term_id );
						if ( !$instance['title'] ) {
							$instance['title'] = $term->name;
						}


						if ( !isset( $cat ) ) {
							$cat = $term;
						}
					}
				} else {
					return false;
				}
			} else {
				add_filter( 'get_terms', array( &$this, 'convert_terms_to_list' ) );
				$terms = get_terms( $taxonomy );
				remove_filter( 'get_terms', array( &$this, 'convert_terms_to_list' ) );
			}
		} else {
			$terms = $instance[$taxonomy];
			$cat = get_term( $instance[$taxonomy], $taxonomy );
		}

		if ( is_object( $cat ) && isset( $cat->term_id ) and $instance['link_cat'] ) {
			$title_link = '<a href="' . get_category_link( $cat->term_id ) . '">';
			$title_close_link = '</a>';
		} else {
			$title_link = $title_close_link = '';
		}

		/* Use the widget title if set, or use the category title. */
		if ( $instance['title'] && !empty( $instance['title'] ) ) {
			$title_name = $instance['title'];
		} elseif ( is_object( $cat ) ) {
			$title_name = $cat->name;
		}

		$args = array(
			'post_type' => $instance['post_types'],
			'posts_per_page' => $instance['num_posts'],
			'showposts' => $instance['num_posts'],
			'orderby' => $instance['orderby'],
			'order' => $instance['order'],
			'tax_query' => array(
				array(
					'taxonomy' => $taxonomy,
					'field' => 'id',
					'terms' => $terms
				) ) );


		if ( $instance['hide_current'] ) {
			$args['post__not_in'] = array( $post->ID );
		}

		$sidebar_query = new WP_Query( $args );

		$excerpt_args = array(
			'length' => $instance['excerpt_length'],
			'excerpt_more_separator' => $instance['excerpt_more_separator'],
			'excerpt_more' => $instance['excerpt_more'],
			'link_excerpt_more' => $instance['link_excerpt_more'],
		);

		/* Start the sidget */
		echo $before_widget;

		/* Display widget title */
		if ( isset( $title_name ) ) {
			echo $before_title . $title_link . $title_name . $title_close_link . $after_title;
		}

		if ( isset( $cat->description ) and $instance['show_description'] ) {
			echo '<p class="sidebar-taxonomy-description">' . $cat->description . '</p>';
		}

		include ($this->get_template( $instance['style'], '/templates/' ));

		echo $after_widget;
	}

	function convert_terms_to_list( $terms ) {
		$list = array( );

		foreach ( $terms as $term ) {
			array_push( $list, $term->term_id );
		}
		return $list;
	}

	function get_widget_templates() {
		$templates = array( );

		if ( $handle = @opendir( get_stylesheet_directory() . '/pretty-sidebar-categories/templates' ) ) {
			while ( false !== ($file = readdir( $handle )) ) {
				if ( $file != "." && $file != ".." && $file != 'list.php' && preg_match( '/.php/', $file ) ) {
					$file = str_replace( '.php', '', $file );
					$name = str_replace( '-', ' ', $file );
					$templates[$file] = ucfirst( $name ) . "<br>";
				}
			}

			closedir( $handle );
		}

		if ( $handle = opendir( $this->pluginPath . '/templates' ) ) {
			while ( false !== ($file = readdir( $handle )) ) {
				if ( $file != "." && $file != ".." && $file != 'list.php' && preg_match( '/.php/', $file ) ) {
					$file = str_replace( '.php', '', $file );
					$name = str_replace( '-', ' ', $file );
					$templates[$file] = ucfirst( $name ) . "<br>";
				}
			}

			closedir( $handle );
		}

		return $templates;
	}

	/**
	 * Retrieve a template file from either the theme or the plugin directory.
	 *
	 * @param <string> $template    The name of the template.
	 * @return <string>             The full path to the template file.
	 */
	function get_template( $template = NULL, $folder = '', $ext = '.php', $type = 'path' ) {
		if ( $template == NULL ) {
			return false;
		}

		$themeFile = get_stylesheet_directory() . '/pretty-sidebar-categories/' . $folder . $template . $ext;

		if ( file_exists( $themeFile ) ) {
			if ( $type == 'url' ) {
				$file = get_bloginfo( 'template_url' ) . '/pretty-sidebar-categories/' . $folder . $template . $ext;
			} else {
				$file = get_stylesheet_directory() . '/pretty-sidebar-categories/' . $folder . $template . $ext;
			}
		} elseif ( $type == 'url' ) {
			$file = $this->templatesURL . $template . $ext;
		} else {
			$file = $this->templatesPath . $template . $ext;
		}

		return $file;
	}

	/**
	 * Display the list of contributors.
	 * @return boolean
	 */
	function contributor_list() {

		if ( function_exists( 'simplexml_load_file' ) ) {
			$this->showFields = array( 'NAME', 'LOCATION', 'COUNTRY' );
			print '<ul>';

			$items = simplexml_load_file( 'http://cdn.grandslambert.com/xml/pretty-sidebar-categories.xml' );

			foreach ( $items as $item ) {
				print '<li>';
				if ( empty( $item->url ) ) {
					print $item->name;
				} else {
					print '<a href="' . $item->url . '" target="_blank">' . $item->name . '</a>';
				}

				if ( !empty( $item->location ) ) {
					print ' from ' . $item->location;
				}

				if ( !empty( $item->country ) ) {
					print ', ' . $item->country;
				}

				print ' contributed ' . $item->item . ' on ' . date( 'F jS, Y', strtotime( $item->date ) ) . '</li>';
			}
			print '</ul>';
		} else {
			_e( 'PHP 5 Required to see a list of contributors.', 'pretty-sidebar-categories' );
		}
	}

	/**
	 * Displays any data sent in textareas.
	 *
	 * @param <type> $input
	 */
	function debug( $input ) {
		$contents = func_get_args();

		foreach ( $contents as $content ) {
			print '<textarea style="width:49%; height:250px; float: left;">';
			print_r( $content );
			print '</textarea>';
		}

		echo '<div style="clear: both"></div>';
	}

}

add_action( 'widgets_init', create_function( '', 'return register_widget("PrettySidebarCategories");' ) );

/**
 * New excerpt template tag
 */
if ( !function_exists( 'gs_the_excerpt' ) ) {

	function gs_the_excerpt( $args = array( ) ) {
		echo gs_get_the_excerpt( $args );
	}

	function gs_get_the_excerpt( $args = array( ) ) {
		global $post;

		$defaults = array(
			'length' => 55,
			'excerpt_more_separator' => '',
			'excerpt_more' => '...',
			'link_excerpt_more' => false,
			'allowed-tags' => 'p'
		);

		$args = wp_parse_args( $args, $defaults );

		$allowed_tags_formatted = '';

		$tags = explode( ',', $args['allowed-tags'] );

		foreach ( $tags as $tag ) {
			$allowed_tags_formatted.= '<' . $tag . '>';
		}

		$text = str_replace( ']]>', ']]&gt;', $post->post_content );
		$text = strip_tags( $text, $allowed_tags_formatted );
		$text = preg_replace( '@<script[^>]*?>.*?</script>@si', '', $text );
		$words = explode( ' ', $text, $args['length'] + 1 );
		if ( count( $words ) > $args['length'] ) {
			array_pop( $words );

			if ( $args['link_excerpt_more'] ) {
				array_push( $words, ' ' . $args['excerpt_more_separator'] . ' <a href="' . get_permalink( $post->ID ) . '">' . $args['excerpt_more'] . '</a>' );
			} else {
				array_push( $words, ' ' . $args['excerpt_more_separator'] . ' ' . $args['excerpt_more'] );
			}
			$text = implode( ' ', $words );
		}

		$tags = explode( ',', $args['allowed-tags'] );
		foreach ( $tags as $tag ) {
			$text.= '</' . $tag . '>';
		}

		return $text;
	}

}

/* Add missing WordPress functions */
if ( !function_exists( 'wp_dropown_taxomomies' ) ) {

	function wp_dropdown_taxonomies( $args = array( ) ) {
		$defaults = array(
			'selected' => 'category',
			'name' => 'taxonomy-select',
			'id' => 'taxonomy_select',
			'onchange' => '',
			'echo' => true,
		);
		$args = wp_parse_args( $args, $defaults );

		$taxonomies = get_taxonomies( array( ), 'object' );

		$output = '<select name="' . $args['name'] . '" id="' . $args['id'] . '"';

		if ( $args['onchange'] ) {
			$output.= 'onchange="' . $args['onchange'] . '"';
		}

		$output.= '>';

		foreach ( $taxonomies as $taxonomy => $details ) {
			if ( $taxonomy != 'nav_menu' and wp_count_terms( $taxonomy, array( 'hide_empty' => true ) ) > 0 ) {
				$output.= '<option value="' . $taxonomy . '" ' . selected( $args['selected'], $taxonomy, false ) . '>' . $details->labels->singular_name . '</option>';
			}
		}

		$output.= '</select>';

		$output = apply_filters( 'wp_dropdown_taxonomies', $output );

		if ( $args['echo'] ) {
			echo $output;
		} else {
			return $output;
		}
	}

}